#include <stdio.h>
#include "xil_io.h"
#define CUSTOM_IP_BASEADDR 0x43C00000
#define REGISTER_0_OFFSET 0x00
#define REGISTER_1_OFFSET 0x04
#define REGISTER_2_OFFSET 0x08
#define REGISTER_3_OFFSET 0x0C

// Function prototypes
void set_custom_ip_register (int baseaddr, int offset, int value);
int get_custom_ip_register(int baseaddr, int offset);

void set_custom_ip_register (int baseaddr, int offset, int value) {
	Xil_Out32(baseaddr + offset, value);
}

int get_custom_ip_register(int baseaddr, int offset) {
 int temp = 0;

 temp = Xil_In32(baseaddr + offset);
 return (temp);
}

int main (void) {

	int i;
	int errors;
	
	int result_is_temp;
	int result_should_temp;

	int input1[7] = {0x0000002a,
	                 0x00000023,
	                 0x00000007,
	                 0xFFFFFFD6,	// -42
	                 0x00000080,	// 128
	                 0xFFFFFFD6,
	                 0x00000007};
					 
	int input2[7] = {0x00000007,
	                 0x00000005,
	                 0x00000008,
	                 0x00000007,
	                 0x00000000,
	                 0x00000000,
	                 0x00000001};
	
	int result_should[7] = { 0x00000006,
							 0x00000007,
							 0x00000000,
							 0x000001FA,	// -6
							 0x000000FF,	// 255
							 0x00000101,	// -255
							 0x00000007};
	
	int result_is[7];
	
	
	printf("############################\n\r");
	printf("### Divider Test Project ###\n\r");
	printf("############################\n\r\n\r");

	printf("Resetting Divider...\n\r");
	set_custom_ip_register(CUSTOM_IP_BASEADDR, REGISTER_0_OFFSET, 0x00);	// Activate Rst_n
	
	printf("Enabling Divider...\n\r\n\r");
	set_custom_ip_register(CUSTOM_IP_BASEADDR, REGISTER_0_OFFSET, 0x02);	// Deactivate Rst_n
	set_custom_ip_register(CUSTOM_IP_BASEADDR, REGISTER_0_OFFSET, 0x03);	// Enable Divider
	
	printf("Testing operation...\r\n");
	
	errors =0;
	
	for (i=0; i<7; i++) {
		printf("Input1 = %d, Input2 = %d\r\n", input1[i], input2[i]);
		
		set_custom_ip_register(CUSTOM_IP_BASEADDR, REGISTER_1_OFFSET, input1[i]);
		set_custom_ip_register(CUSTOM_IP_BASEADDR, REGISTER_2_OFFSET, input2[i]);
		
		printf("Result = "); 
		
		result_is[i] = get_custom_ip_register(CUSTOM_IP_BASEADDR, REGISTER_3_OFFSET);

		if (result_is[i] < 256)
			printf("%d\r\n", result_is[i]);
		else 
			printf("%d\r\n", result_is[i] - 512);
		
		if (result_is[i] != result_should[i]) {
			errors+=1;

			if (result_is[i] < 256)
				result_is_temp = result_is[i];
			else
				result_is_temp = result_is[i] - 512;

			if (result_should[i] < 256)
				result_should_temp = result_should[i];
			else
				result_should_temp = result_should[i] - 512;

			printf("Error detected at i=%d, SHOULD: %d, IS: %d\r\n", i, result_should_temp, result_is_temp);
		}
		else
		printf("\r\n", result_is[i]);
	}

	if (errors > 0) {
		printf("Testing IP finished.\r\n\r\n ### FAILED! ###\r\n");
		printf("Number of errors: %d\r\n", errors);
	}
	else
		printf("Testing IP finished.\r\n\r\n ### SUCCESS! ###\r\n");


	return 0;

}







