
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

use std.textio.all;

ENTITY axi_divider_tb IS
END axi_divider_tb;
 
ARCHITECTURE behavior OF axi_divider_tb IS 
	
	component axi_divider_0 IS
	  PORT (
		s00_axi_awaddr 	: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0);
		s00_axi_awprot 	: IN	STD_LOGIC_VECTOR(2 DOWNTO 0);
		s00_axi_awvalid : IN 	STD_LOGIC;
		s00_axi_awready : OUT 	STD_LOGIC;
		s00_axi_wdata 	: IN 	STD_LOGIC_VECTOR(31 DOWNTO 0);
		s00_axi_wstrb 	: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0);
		s00_axi_wvalid 	: IN 	STD_LOGIC;
		s00_axi_wready 	: OUT 	STD_LOGIC;
		s00_axi_bresp 	: OUT 	STD_LOGIC_VECTOR(1 DOWNTO 0);
		s00_axi_bvalid 	: OUT 	STD_LOGIC;
		s00_axi_bready 	: IN 	STD_LOGIC;
		s00_axi_araddr 	: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0);
		s00_axi_arprot 	: IN 	STD_LOGIC_VECTOR(2 DOWNTO 0);
		s00_axi_arvalid : IN 	STD_LOGIC;
		s00_axi_arready : OUT 	STD_LOGIC;
		s00_axi_rdata 	: OUT 	STD_LOGIC_VECTOR(31 DOWNTO 0);
		s00_axi_rresp 	: OUT 	STD_LOGIC_VECTOR(1 DOWNTO 0);
		s00_axi_rvalid 	: OUT 	STD_LOGIC;
		s00_axi_rready 	: IN 	STD_LOGIC;
		s00_axi_aclk 	: IN 	STD_LOGIC;
		s00_axi_aresetn : IN 	STD_LOGIC
	  );
	END component;
	
	component axi_traffic_gen_0 is
	  Port ( 
		s_axi_aclk 				: in 	STD_LOGIC;
		s_axi_aresetn 			: in 	STD_LOGIC;
		m_axi_lite_ch1_awaddr 	: out 	STD_LOGIC_VECTOR ( 31 downto 0 );
		m_axi_lite_ch1_awprot 	: out 	STD_LOGIC_VECTOR ( 2 downto 0 );
		m_axi_lite_ch1_awvalid 	: out 	STD_LOGIC;
		m_axi_lite_ch1_awready 	: in 	STD_LOGIC;
		m_axi_lite_ch1_wdata 	: out 	STD_LOGIC_VECTOR ( 31 downto 0 );
		m_axi_lite_ch1_wstrb 	: out 	STD_LOGIC_VECTOR ( 3 downto 0 );
		m_axi_lite_ch1_wvalid 	: out 	STD_LOGIC;
		m_axi_lite_ch1_wready 	: in 	STD_LOGIC;
		m_axi_lite_ch1_bresp 	: in 	STD_LOGIC_VECTOR ( 1 downto 0 );
		m_axi_lite_ch1_bvalid 	: in 	STD_LOGIC;
		m_axi_lite_ch1_bready 	: out 	STD_LOGIC;
		m_axi_lite_ch1_araddr 	: out 	STD_LOGIC_VECTOR ( 31 downto 0 );
		m_axi_lite_ch1_arvalid 	: out 	STD_LOGIC;
		m_axi_lite_ch1_arready 	: in 	STD_LOGIC;
		m_axi_lite_ch1_rdata 	: in 	STD_LOGIC_VECTOR ( 31 downto 0 );
		m_axi_lite_ch1_rvalid 	: in 	STD_LOGIC;
		m_axi_lite_ch1_rresp 	: in 	STD_LOGIC_VECTOR ( 1 downto 0 );
		m_axi_lite_ch1_rready 	: out 	STD_LOGIC;
		done 					: out 	STD_LOGIC;
		status 					: out 	STD_LOGIC_VECTOR ( 31 downto 0 )
	  );
	end component;

	constant AXI_LITE_ADDR_WIDTH : integer := 32;
	constant AXI_LITE_DATA_WIDTH : integer := 32;
	
   --Input
	signal clk						: STD_LOGIC := '0';
	signal rstn						: STD_LOGIC;
	
	signal s00_axi_awaddr 	: STD_LOGIC_VECTOR(3 DOWNTO 0);
	signal s00_axi_awprot 	: STD_LOGIC_VECTOR(2 DOWNTO 0);
	signal s00_axi_awvalid	: STD_LOGIC;
	signal s00_axi_awready	: STD_LOGIC;
	signal s00_axi_wdata 	: STD_LOGIC_VECTOR(31 DOWNTO 0);
	signal s00_axi_wstrb 	: STD_LOGIC_VECTOR(3 DOWNTO 0);
	signal s00_axi_wvalid 	: STD_LOGIC;
	signal s00_axi_wready 	: STD_LOGIC;
	signal s00_axi_bresp 	: STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal s00_axi_bvalid 	: STD_LOGIC;
	signal s00_axi_bready 	: STD_LOGIC;
	signal s00_axi_araddr 	: STD_LOGIC_VECTOR(3 DOWNTO 0);
	signal s00_axi_arprot 	: STD_LOGIC_VECTOR(2 DOWNTO 0);
	signal s00_axi_arvalid	: STD_LOGIC;
	signal s00_axi_arready	: STD_LOGIC;
	signal s00_axi_rdata 	: STD_LOGIC_VECTOR(31 DOWNTO 0);
	signal s00_axi_rresp 	: STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal s00_axi_rvalid 	: STD_LOGIC;
	signal s00_axi_rready 	: STD_LOGIC;
	signal s00_axi_aclk 	: STD_LOGIC := '0';	
	signal s00_axi_aresetn	: STD_LOGIC;
	
	signal m_axi_lite_ch1_araddr, m_axi_lite_ch1_awaddr : STD_LOGIC_VECTOR(31 DOWNTO 0);
	
	signal done				: std_logic;
	signal status			: STD_LOGIC_VECTOR ( 31 downto 0 );

	BEGIN

	s00_axi_arprot <= (others => '0');
	
	axi_divider_0_inst: axi_divider_0
	  PORT MAP(
		s00_axi_aclk 	=> s00_axi_aclk 	,
		s00_axi_aresetn => s00_axi_aresetn,
		s00_axi_awaddr 	=> s00_axi_awaddr ,
		s00_axi_awprot 	=> s00_axi_awprot ,
		s00_axi_awvalid => s00_axi_awvalid,
		s00_axi_awready => s00_axi_awready,
		s00_axi_wdata 	=> s00_axi_wdata ,
		s00_axi_wstrb 	=> s00_axi_wstrb ,
		s00_axi_wvalid 	=> s00_axi_wvalid ,
		s00_axi_wready 	=> s00_axi_wready ,
		s00_axi_bresp 	=> s00_axi_bresp ,
		s00_axi_bvalid 	=> s00_axi_bvalid ,
		s00_axi_bready 	=> s00_axi_bready ,
		s00_axi_araddr 	=> s00_axi_araddr ,
		s00_axi_arvalid => s00_axi_arvalid,
		s00_axi_arready => s00_axi_arready,
		s00_axi_rdata 	=> s00_axi_rdata ,
		s00_axi_rvalid 	=> s00_axi_rvalid ,
		s00_axi_rresp 	=> s00_axi_rresp ,
		s00_axi_rready 	=> s00_axi_rready,
		
		s00_axi_arprot 	=> s00_axi_arprot
	 );
	 
	 s00_axi_araddr <= m_axi_lite_ch1_araddr(3 downto 0);
	 s00_axi_awaddr <= m_axi_lite_ch1_awaddr(3 downto 0);
	  
	axi_traffic_gen_0_inst : axi_traffic_gen_0
	  Port MAP( 
		s_axi_aclk 				=> s00_axi_aclk 	,
		s_axi_aresetn 			=> s00_axi_aresetn,
		m_axi_lite_ch1_awaddr 	=> m_axi_lite_ch1_awaddr ,
		m_axi_lite_ch1_awprot 	=> s00_axi_awprot ,
		m_axi_lite_ch1_awvalid 	=> s00_axi_awvalid,
		m_axi_lite_ch1_awready 	=> s00_axi_awready,
		m_axi_lite_ch1_wdata 	=> s00_axi_wdata ,
		m_axi_lite_ch1_wstrb 	=> s00_axi_wstrb ,
		m_axi_lite_ch1_wvalid 	=> s00_axi_wvalid ,
		m_axi_lite_ch1_wready 	=> s00_axi_wready ,
		m_axi_lite_ch1_bresp 	=> s00_axi_bresp ,
		m_axi_lite_ch1_bvalid 	=> s00_axi_bvalid ,
		m_axi_lite_ch1_bready 	=> s00_axi_bready ,
		m_axi_lite_ch1_araddr 	=> m_axi_lite_ch1_araddr ,
		m_axi_lite_ch1_arvalid 	=> s00_axi_arvalid,
		m_axi_lite_ch1_arready 	=> s00_axi_arready,
		m_axi_lite_ch1_rdata 	=> s00_axi_rdata ,
		m_axi_lite_ch1_rvalid 	=> s00_axi_rvalid ,
		m_axi_lite_ch1_rresp 	=> s00_axi_rresp ,
		m_axi_lite_ch1_rready 	=> s00_axi_rready,
		done 					=> done,
		status 					=> status
	  );
	  
   CLK_GEN : process(clk)
	begin
		clk <= not clk after 5 ns;
	end process;
 
	s00_axi_aclk <= clk;
	s00_axi_aresetn <= rstn;
 
   stim_proc: process
   
   begin		
		wait until rising_edge(clk);
		rstn        <= '0';
		
		wait for 100 ns;
		rstn        <= '1';
				
		wait;
	end process;
		
END;
