
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

use std.textio.all;

ENTITY div16_tb IS
generic(
		IMAGE_WIDTH  				: INTEGER range 3 to 2047 := 640;
		IMAGE_HEIGHT 				: INTEGER range 3 to 2047 := 480;
		ADDR_WIDTH   				: POSITIVE := 13;		--Blockram line_buffer
		DATA_WIDTH   				: POSITIVE := 1;		--Blockram line_buffer
		PIXEL_COUNT  				: POSITIVE := 6;    		--filter_major
	--	ROW_LENGTH					: POSITIVE :=640;		--region detect
		ROW_LENGTH_WIDTH			: POSITIVE := 10;		--region detect
	--	NUMBER_OF_ROWS				: POSITIVE :=480;		--region detect
		ROW_NUMBER_WIDTH			: POSITIVE := 9;		--region detect
		FEATURE_BRAM_ADDR_WIDTH     : POSITIVE :=13	--region detect
	
    );
END div16_tb;
 
ARCHITECTURE behavior OF div16_tb IS 
   	
    COMPONENT div16_8_8 is
	port (
		clk        : in  STD_LOGIC;
		en         : in  STD_LOGIC;
		rstn       : in  STD_LOGIC;
		a          : in  STD_LOGIC_VECTOR(16 downto 0);
		b          : in  STD_LOGIC_VECTOR( 7 downto 0);
		result     : out STD_LOGIC_VECTOR( 8 downto 0)		
	);
	end COMPONENT;
    

   --Input
	signal clk						: STD_LOGIC := '0';
	signal en						: STD_LOGIC;
	signal rstn						: STD_LOGIC;
	
	signal input_a          : STD_LOGIC_VECTOR(16 downto 0);
	signal input_b          : STD_LOGIC_VECTOR( 7 downto 0);
	signal result     : STD_LOGIC_VECTOR( 8 downto 0);
	
	signal cnt		: integer range 0 to 8;
	
	type input_a_arr_type is array (natural range <>) of  STD_LOGIC_VECTOR(16 downto 0);
	type input_b_arr_type is array (natural range <>) of  STD_LOGIC_VECTOR(7 downto 0);
	
	constant input_a_arr : input_a_arr_type (0 to 7) := (  std_logic_vector(to_signed(	42, 17)), 
															   std_logic_vector(to_signed(	35, 17)),
															   std_logic_vector(to_signed(	 7, 17)),
															   std_logic_vector(to_signed( -42, 17)),
															   std_logic_vector(to_signed( 255, 17)), 
															   std_logic_vector(to_signed( 128, 17)),
															   std_logic_vector(to_signed( -42, 17)),												
															   std_logic_vector(to_signed(	 7,  17))
	);
	constant input_b_arr : input_b_arr_type (0 to 7) :=  ( std_logic_vector(to_unsigned( 7 , 8)), 
															   std_logic_vector(to_unsigned( 5 , 8)),
															   std_logic_vector(to_unsigned( 8 , 8)),
															   std_logic_vector(to_unsigned( 7 , 8)),
															   std_logic_vector(to_unsigned( 8 , 8)),
															   std_logic_vector(to_unsigned( 0 , 8)),
															   std_logic_vector(to_unsigned( 0 , 8)),
															   std_logic_vector(to_unsigned( 1 , 8))
	);

	BEGIN
	
	uut: div16_8_8
	port map(
		clk        => clk   ,
		en         => en    ,
		rstn       => rstn  ,
		a          => input_a,
		b          => input_b,
		result     => result	
	);
	
 	
		

		
   CLK_GEN : process(clk)
	begin
		clk <= not clk after 5 ns;
	end process;
 
   stim_proc: process

   
   begin		
		wait until rising_edge(clk);
		rstn        <= '0';
		en     <= '0';
		
		cnt <= 0;
		
		wait for 100 ns;
		rstn        <= '1';
		
		en  <= '1';
		while cnt < 8 loop
			input_a <= input_a_arr(cnt);
			input_b <= input_b_arr(cnt);
			cnt <= cnt+1;
			
			wait until rising_edge(clk);
		end loop;
		
		wait;
	end process;
		
END;
